// ---------------------------------------------------------
// makeComplex - Given 'real' and 'imaginary', return a
//               complex number object.
// ---------------------------------------------------------

function makeComplex(real, imaginary)
{
   var mReal      = real;
   var mImaginary = imaginary;

   var theObject = 
   {
      real:      function() { return(mReal);      },
      imaginary: function() { return(mImaginary); },

      square: function()
      {
         var r = mReal * mReal - mImaginary * mImaginary;
         var i = 2 * mReal * mImaginary;

         return(makeComplex(r, i));
      },

      add: function(other)
      {
         return(
            makeComplex(
               mReal + other.real(), mImaginary + other.imaginary()
            )
         );
      },

      magnitude: function()
      {
         return(Math.sqrt(mReal * mReal + mImaginary * mImaginary));
      },

      delta: function()
      {
         return(
            Math.abs(Math.abs(mReal) - Math.abs(mImaginary))
         );
      },

      show: function()
      {
         alert("x: " + mReal + ", y: " + mImaginary);
      }
   };

   return(theObject);
}

// ---------------------------------------------------------
// getSinColor - Map 'value' to a color from 0 to 255, note
//               the colors "wrap around" because we use the
//               'sin' function (which causes 'getSinColor'
//               to return the same value when value = 0
//               as it does when value = 1).
// ---------------------------------------------------------

function getSinColor(value)
{
   return(127.5 + 127.5 * Math.sin(value * Math.PI * 2.0));
}

// ---------------------------------------------------------
// setForID - Sets 'innerHTML' for the element with 'id'
// ---------------------------------------------------------

function setForID(id, value)
{
   var item = document.getElementById(id);

   if(item)
      item.innerHTML = value;
}

// ---------------------------------------------------------
// drawFractal - Render our mandelbrot image
// ---------------------------------------------------------

function drawFractal()
{
   var canvas  = document.getElementById('canvas');
   var context = canvas.getContext('2d');

   var width  = context.canvas.width;
   var height = context.canvas.height;

   var imgd = context.createImageData(width, 1);

   var xMin = -2.0;
   var yMin = -2.0;  
   var yMax =  2.0;
   var xMax =  2.0;

   var xStep = (xMax - xMin) / width;
   var yStep = (yMax - yMin) / height;

   var start = Date.now();
   var y     = yMin;

   for(var i = 0; i < height; i ++)
   {
      var x = xMin;

      for(var j = 0, k = 0; j < width; j ++)
      {
         var c = makeComplex(x, y);
         var z = makeComplex(0, 0);

         var count = 0;

         var v1max = 0.0;
         var v1tot = 0.0;

         var v2max = 0.0;
         var v2tot = 0.0;

         var v3max = 0.0;
         var v3tot = 0.0;

         while(z.magnitude() <= 1000 && ++count < 25)
         {
            z = z.square().add(c);

            var value = makeComplex(z.real(), z.imaginary());
            var v1    = value.delta();

            value  = value.square();
            var v2 = value.delta();

            value  = value.square();
            var v3 = value.delta();

            if(v1 > v1max) v1max = v1;
            if(v2 > v2max) v2max = v2;
            if(v3 > v3max) v3max = v3;

            v1tot += v1;
            v2tot += v2;
            v3tot += v3;
         }

         var rv = 0;
         var gv = 0;
         var bv = 0;

         if(count > 0)
         {
            var v1avg = v1tot / count;
            var v2avg = v2tot / count;
            var v3avg = v3tot / count;

            rv = (v1max - v1avg) / v1max;
            gv = (v2max - v2avg) / v2max;
            bv = (v3max - v3avg) / v3max;
         }
      
         var r = getSinColor(rv);
         var g = getSinColor(gv);
         var b = getSinColor(bv + .2);

         var rgb = (r + g + b) / 3.0;

         var f1 = 0.3;
         var f2 = 1.0 - f1;

         r = f1 * r + f2 * rgb;
         g = f1 * g + f2 * rgb;
         b = f1 * b + f2 * rgb;

         imgd.data[k++] = r;
         imgd.data[k++] = g;
         imgd.data[k++] = b;
         imgd.data[k++] = 255;  // alpha

         x += xStep;
      }

      y += yStep;

      context.putImageData(imgd, 0, i);
   }

   var elapsed = (Date.now() - start) / 1000.0;

   setForID(
      "message", "Finished (Elapsed Time: " + elapsed + " Seconds)"
   );

   setForID("form-td", "");
}
